﻿using Perception.Interfaces;
using System.Runtime.Serialization;
using System.Windows.Forms;
using Perception.Common;
using Perception.Components;
using Perception.UserKeys;
using TimeView;

namespace Perception.ScriptAction.Training
{
    /// <summary>
    /// User script action class which can be used by the Perception User-keys
    /// </summary>
    public class TraceAutoScaleAction : IScriptAction, IScriptActionInfo, IConfigurable
    {
        #region Private members
        private const string FIELDNAME_VARIABLE_1 = "FieldNameVariable1";   // Example local string variable field name for Persistency
        private string m_strVariable1 = string.Empty;                       // Example local string variable
        #endregion

        #region Constructors
        /// <summary>
        /// Constructor
        /// </summary>
        public TraceAutoScaleAction()
        {
            m_strVariable1 = string.Empty;
        }

        /// <summary>
        /// Serialization constructor
        /// </summary>
        /// <param name="info">Serialization info</param>
        /// <param name="context">Streaming context</param>
        public TraceAutoScaleAction(SerializationInfo info, StreamingContext context)
        {
            m_strVariable1 = Tools.GetValue<string>(info, FIELDNAME_VARIABLE_1, string.Empty);
        }
        #endregion

        #region IScriptAction Members
        /// <summary>
        /// The RunAction is called when the script has to be executed
        /// </summary>
        /// <param name="context"></param>
        /// <returns></returns>
        public ScriptActionResult RunAction(IScriptContext context)
        {
            string cError = "";
            if (!ExecuteAction(context.OwnerWindow, out cError))
            {
                return HandleError(context, cError, ScriptActionResult.ErrorAbort);
            }
            return ScriptActionResult.OK;
        }
        #endregion

        #region private methods
        private ScriptActionResult HandleError(IScriptContext context, string message, ScriptActionResult actionResult)
        {
            if (context != null)
                actionResult = context.NotifyUser(this, message, actionResult);
            else
                actionResult = ScriptActionResult.ErrorAbort;
            return actionResult;
        }

        private bool ExecuteAction(IWin32Window owner, out string cError)
        {
            Display activeDisplay = DisplayHelper.GetActiveDisplay();
            if (activeDisplay == null)
            {
                cError = "Can not perform this action because no active display is found";
                return false;
            }

            ActionResultType aResult = DisplayHelper.DoAction(activeDisplay,
                                TimeView.ActionType.ActionType_TraceAutoScale);

            if (aResult == ActionResultType.ActionResultType_OK)
            {
                cError = "";
                return true;
            }

            cError = string.Format("Auto trace scale failed: Action result code: {0}", aResult);
            return false;
        }
        #endregion

        #region ISerializable Members
        /// <summary>
        /// Method to serialize information of this script
        /// </summary>
        /// <param name="info"></param>
        /// <param name="context"></param>
        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue(FIELDNAME_VARIABLE_1, m_strVariable1);
        }

        #endregion

        #region IScriptActionInfo Members
        /// <summary>
        /// The text of this script action, this text is shown in the various selection menus of scripts in Perception
        /// </summary>
        public string Text
        {
            get { return Properties.Resources.ScriptText; }
        }

        /// <summary>
        /// The category to which this script action belongs
        /// </summary>
        public string Category
        {
            get { return Properties.Resources.ScriptCategory; }
        }

        /// <summary>
        /// The help text of the script action
        /// </summary>
        public string HelpText
        {
            get { return Properties.Resources.ScriptHelpText; }
        }

        /// <summary>
        /// Picture used for this script action 
        /// </summary>
        public System.Drawing.Image Image
        {
            get { return Properties.Resources.UserScript_32x32; }
        }
        #endregion

        #region IConfigurable Members

        /// <summary>
        /// Call the dialog for configuring the action parameters
        /// </summary>
        /// <param name="owner"></param>
        /// <returns></returns>
        public DialogResult Configure(IWin32Window owner)
        {
            using (UserScriptActionConfigurationDialog Dialog = new UserScriptActionConfigurationDialog())
            {
                Dialog.txtVariable1.Text = m_strVariable1;
                DialogResult result = Dialog.ShowDialog(owner);
                if (result == DialogResult.OK)
                {
                    m_strVariable1 = Dialog.txtVariable1.Text;
                }
            }
            return DialogResult.OK;
        }

        #endregion
    }
}
