using System;
using System.Collections.Generic;
using System.Text;

using Perception.FormulaDatabase.Functions;
using Perception.FormulaDatabase.FunctionSupport;
using RecordingInterface;

namespace FormulaDBFunctions
{
    public class MyFunctionInfo : IFunctionInfo
    {
        #region IFunctionInfo Members

        public string Category
        {
            get
            {
                return "CSI Functions";
            }
        }

        public IFunction CreateFunction()
        {
            // for testing purposes as long as the function
            // is not implemented: return null;
            return new CreateWaveformFromAverage();
        }

        public string Description
        {
            get
            {
                return "Creates a DC waveform with value of average of the waveform to process (InWave).";
            }
        }

        public string Example
        {
            get
            {
                return "@CSI_MeanWave(Active.Group1.Recorder1.Ch_A2)";
            }
        }

        public int MinimumParameterCount
        {
            get
            {
                return 1;
            }
        }

        public string Name
        {
            get
            {
                return "CSI_MeanWave";
            }
        }

        public string[] ParameterDescriptions
        {
            get
            {
                string[] ParamDescriptions = { "The waveform to process" };
                return ParamDescriptions;
            }
        }

        public string[] ParameterNames
        {
            get
            {
                string[] ParamNames = { "InWave" };
                return ParamNames;
            }
        }

        public DataSrcManager.PoolEntryType[] ParameterTypes
        {
            get
            {
                DataSrcManager.PoolEntryType[] PETypes = { DataSrcManager.PoolEntryType.PoolEntryType_Waveform };
                return PETypes;
            }
        }

        #endregion
    }

    class AverageWrapperSegment : SampleBySampleDataSegment
    {
        // this class uses the "SampleBySampleDataSegment" as base class
        // it creates a waveform segment that is a DC value
        // DC = a calculated average value

        // internal member
        double m_DeAverageValue;

        // constructor, requires a data segment, the average value
        // and uses base class for "difficult" stuff
        public AverageWrapperSegment(IDataSegment OriginalSegment, double DeAverageValue)
             : base(OriginalSegment)
        {
            // copy average value to internal
            m_DeAverageValue = DeAverageValue;
        }

        public override void ProcessSamples(double[] fSamples)
        {
            // override the process samples method.
            // This one gets a segment of samples in a double array
            for (int nSample = 0; nSample < fSamples.Length; nSample++)
            {
                // replace each sample with the average value
                fSamples[nSample] = m_DeAverageValue;
            }
        }
        public override void DisplayRange(out double DisplayFrom, out double DisplayTo)
        {
            base.DisplayRange(out DisplayFrom, out DisplayTo);
            // DisplayFrom = base.DisplayFrom;
            // DisplayTo = base.DisplayTo;
            if (DisplayFrom < m_DeAverageValue)
                DisplayFrom = 2 * m_DeAverageValue;
            if (DisplayTo > m_DeAverageValue)
                DisplayTo = -2 * m_DeAverageValue;
        }
    }

    public class CreateWaveformFromAverage : SampleBySampleOrNumericalFunction
    {
        // this class uses the "SampleBySampleOrNumericalFunction"
        // it creates a complete waveform that is a DC value
        // The DC value = a calculated average value

        double m_Average = double.NaN;

        // override the CreateWrapperSegment
        // it creates an interim data segment, calculates the average
        // and passes it to the interim data segment 
        protected override SampleBySampleDataSegment CreateWrapperSegment(IDataSegment OriginalSegment)
        {
            lock (this)
            {
                CalculateAverageOfSource();
                return new AverageWrapperSegment(OriginalSegment, m_Average);
            }
        }

        public override void Reset()
        {
            lock (this)
            {
                m_Average = double.NaN;
            }
        }

        private void CalculateAverageOfSource()
        {
            // if m_Average is not NaN then it has already be calculated before, 
            // don't do it again end return immediately.
            if (!double.IsNaN(m_Average))
                return;

            // m_Parameters array is part of the inherited Functionbase
            // m_Parameters[0] is the data source
            IDataSrc myDS = m_Parameters[0];

            // calculate average

            // use only analog waveforms for the this example
            if (myDS.DataType != DataSourceDataType.DataSourceDataType_AnalogWaveform)
                return;

            // initialize variables
            object Result;
            IDataSegments Segments;

            // fetch data
            myDS.Data(myDS.Sweeps.StartTime, myDS.Sweeps.EndTime, out Result);

            // fetch segments within this data
            Segments = Result as IDataSegments;

            // no segments
            if (Segments.Count == 0)
                return;

            // loop through all segments to find out the average value
            m_Average = 0;
            int iTotalCnt = 0;

            // for all segments
            for (int j = 1; j <= Segments.Count; j++)
            {
                // fetch the data
                int dCnt = Segments[j].NumberOfSamples;
                Segments[j].Waveform(DataSourceResultType.DataSourceResultType_Double64, 1, dCnt, 1, out Result);
                double[] Buffer = (double[])Result;

                // sum all sample values
                for (int m = 0; m < Buffer.Length; m++)
                {
                    m_Average = m_Average + Buffer[m];
                }

                // sum the total number of samples
                iTotalCnt = iTotalCnt + dCnt;
            }

            // calculate the average
            m_Average = m_Average / iTotalCnt;
        }

        protected override object ProcessValue(object Value)
        {
            // override the ProcessValue method and perform calculation
            // on the input value
            CalculateAverageOfSource();
            // temporarily use double as output since double and integer are not accepted by display
            // routine GetValueAtTime
            return (double)m_Average;
        }
    }
}
