#region Using directives
using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.Serialization;

using Perception.Sheets;
using Perception;
using Perception.Common;
using Perception.ILO.Engine;
using Perception.Core.ExtensionMethods;
using Perception.CSI.Support.Sheets;
using Perception.ILO.Engine.Interfaces;
#endregion

namespace HelmetTest
{
    /// <summary>
    /// The Sheet Control class, implementing the ISheet interface
    /// </summary>
    [SheetInfoRetriever(typeof(SheetControlInfoRetriever))]
    [SheetInformationProvider(typeof(SheetControlInfoProvider))]
    public partial class SheetControl : UserControl, ISheet
    {
        #region Members
        private string m_strUserName = Properties.Resources.IDS_USERNAME;
        private IProgram m_iProgram = null;
        private UIState m_UIState = UIState.Invisible;
        private InitializeState m_InitializeState = InitializeState.Unknown;
        private bool m_bDisposed = false;
        #region -> MyMembers
        private CtrlAcquisitionSystem m_MyDemoSystem = null;
        private CtrlGroup m_GroupAll = null;
        private CtrlRecorder m_TheRecorder = null;
        #endregion
        #endregion

        #region Constructor
        public SheetControl()
        {
            InitializeComponent();
            Disposed += SheetDisposed;
        }
        #endregion

        #region ISheet Members
        #region -> Events
        /// <summary>
        /// Occurs when the user name of the sheet is changed.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler UserNameChanged;

#pragma warning disable 067
        /// <summary>
        /// Occurs when one or more tool bar items are updated.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler ToolItemsUpdated;

        /// <summary>
        /// Occurs when the sheet asks to rebuild the menu and tool bar.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler RebuildDynamicMenuRequested;
#pragma warning restore 067
        #endregion

        #region -> Properties
        /// <summary>
        /// Gets/Sets the sheet user-name.
        /// </summary>
        public string UserName
        {
            get
            {
                // Return the current UserName
                return this.m_strUserName;
            }
            set
            {
                // Compare the new UserName to the current UserName
                if (String.Compare(this.m_strUserName, value) != 0)
                {
                    // Set the new UserName
                    this.m_strUserName = value;

                    // Fire the UserNameChanged event
                    this.UserNameChanged?.Invoke(this, new EventArgs());
                }
            }
        }

        /// <summary>
        /// Gets the name of the dynamic drop-down menu entry for the sheet.
        /// </summary>
        public string MenuName
        {
            get
            {
                if (String.IsNullOrEmpty(this.m_strUserName))
                {
                    // Return a string from the resource table when no current User Name is available
                    return Properties.Resources.IDS_USERNAME;
                }
                else
                {
                    // Return the current User Name
                    return this.m_strUserName;
                }
            }
        }

        /// <summary>
        /// Gets the icon of the sheet.
        /// </summary>
        public Icon Icon
        {
            get
            {
                return Properties.Resources.SheetIcon;
            }
        }

        /// <summary>
        /// Gets whether the delete may be activated. Application delete should delete the current active item on the sheet.
        /// </summary> 
        public bool DeleteActiveItemSupported
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets/Sets the user-interface state of the sheet.
        /// </summary>
        public UIState UIState
        {
            get
            {
                return this.m_UIState;
            }
            set
            {
                this.m_UIState = value;
            }
        }

        /// <summary>
        /// Gets the initialization state of the sheet.
        /// </summary>
        public InitializeState InitializeState
        {
            get
            {
                return this.m_InitializeState;
            }
        }
        #endregion

        #region -> Methods
        /// <summary>
        /// Update the items in the dynamic drop-down menu.
        /// </summary>
        public void UpdateMenuItems()
        {
        }

        /// <summary>
        /// Delete the actively selected item on the sheet.
        /// </summary>
        public void DeleteActiveItem()
        {
        }

        /// <summary>
        /// Perform steps to auto-configure the sheet.
        /// </summary>
        public void AutoConfig()
        {
        }

        /// <summary>
        /// Initialize the sheet.
        /// </summary>
        /// <param name="iProgram">Interface to the program.</param>
        /// <returns>Returns <b>Succeeded</b> if the initialization is successful.</returns>
        public InitializeState Initialize(IProgram iProgram)
        {
            if (iProgram.UserMode == UserMode.Review)
            {
                this.m_InitializeState = InitializeState.NotAllowed;
            }
            else
            {
                try
                {
                    this.m_iProgram = iProgram;
                    RegisterComponents();
                    this.m_MyDemoSystem = CtrlAcquisitionSystemFactory.Create();
                    this.m_GroupAll = m_MyDemoSystem.Groups.GroupAll;
                    HookToGroupAll();
                    this.m_InitializeState = InitializeState.Succeeded;
                }
                catch
                {
                    this.m_InitializeState = InitializeState.NotAllowed;
                    PerceptionMessageBox.Show(this, "Could not initialize sheet", "CSI: Catch", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            return this.m_InitializeState;

        }

        private void HookToGroupAll()
        {
            UnHookFromGroupAll();
            if (m_GroupAll != null)
            {
                m_GroupAll.AcquisitionStateChanged += GroupAllAcquisitionStateChanged;
                m_GroupAll.RecorderAdded += GroupAllRecorderAdded;
            }
        }

        private void UnHookFromGroupAll()
        {
            if (m_GroupAll != null)
            {
                try
                {
                    m_GroupAll.AcquisitionStateChanged -= GroupAllAcquisitionStateChanged;
                    m_GroupAll.RecorderAdded -= GroupAllRecorderAdded;
                }
                catch
                {
                }
            }
        }

        void GroupAllRecorderAdded(object sender, CtrlRecorder Recorder)
        {
            this.InvokeOnUI(() => DoGroupAllRecorderAdded(sender, Recorder));
        }

        void DoGroupAllRecorderAdded(object sender, CtrlRecorder Recorder)
        {
            // is this the recorder we want ?
            if (Recorder.Name != "Recorder A")
                return;

            m_TheRecorder = Recorder;
            // this should be > 0, but you never know
            if (m_TheRecorder.Channels.Count == 0)
            {
                // error handling here
            }
        }

        void GroupAllAcquisitionStateChanged(object sender, int Running, int OneShot, int Stopping, int Paused, int Idle)
        {
            this.InvokeOnUI(() => DoGroupAllAcquisitionStateChanged(sender, Running, OneShot, Stopping, Paused, Idle));
        }

        void DoGroupAllAcquisitionStateChanged(object sender, int Running, int OneShot, int Stopping, int Paused, int Idle)
        {
            // for any acquisition state change disable print unless correctly finished
            PrintCmd.Enabled = false;
            PrintCmd_MenuItem.Enabled = false;
            PrintCmd_ToolStripItem.Enabled = false;

            if (Running > 0)
            {
                StatusArea.Text = "Acquisition: running";
                StartCmd.Enabled = false;
                StartCmd_MenuItem.Enabled = false;
                StartCmd_ToolStripItem.Enabled = false;
            }
            else if (OneShot > 0)
            {
                StatusArea.Text = "Acquisition: single shot";
                StartCmd.Enabled = false;
                StartCmd_MenuItem.Enabled = false;
                StartCmd_ToolStripItem.Enabled = false;
            }
            else if (Stopping > 0)
            {
                StatusArea.Text = "Acquisition: stopping";
                StartCmd.Enabled = false;
                StartCmd_MenuItem.Enabled = false;
                StartCmd_ToolStripItem.Enabled = false;
            }
            else if (Paused > 0)
            {
                StatusArea.Text = "Acquisition: pause";
                StartCmd.Enabled = true;
                StartCmd_MenuItem.Enabled = true;
                StartCmd_ToolStripItem.Enabled = true;
            }
            else if (Idle > 0)
            {
                StatusArea.Text = "Test completed";
                PrintCmd.Enabled = true;
                PrintCmd_MenuItem.Enabled = true;
                PrintCmd_ToolStripItem.Enabled = true;

                StatusArea.Text = "Acquisition: idle";
                StartCmd.Enabled = true;
                StartCmd_MenuItem.Enabled = true;
                StartCmd_ToolStripItem.Enabled = true;
            }
            else
            {
                StatusArea.Text = "****";
                StartCmd.Enabled = false;
                StartCmd_MenuItem.Enabled = false;
                StartCmd_ToolStripItem.Enabled = false;
            }
            // Fire the ToolItemsUpdated event to update the toolbar
            if (this.ToolItemsUpdated != null)
                this.ToolItemsUpdated(this, new EventArgs());
        }

        /// <summary>
        /// Perform steps that should be taken after a new settings file was loaded.
        /// </summary>
        public void PostLoad()
        {
        }

        /// <summary>
        /// Collect the dynamic drop-down menu.
        /// </summary>
        /// <returns>A tool strip array containing the items in the dynamic drop-down menu.</returns>
        public ToolStripItem[] GetDynamicMenu()
        {
            ContextMenuStrip strip = contextMenuStrip1;
            if (this.IsDisposed || this.Disposing || strip == null)
                return null;
            ToolStripItem[] Result = new ToolStripItem[strip.Items.Count];
            strip.Items.CopyTo(Result, 0);
            return Result;
        }

        /// <summary>
        /// Collect the dynamic tool bar menu.
        /// </summary>
        /// <returns>A tool strip array containing the items in the dynamic tool-bar menu.</returns>
        public ToolStripItem[] GetDynamicToolBar()
        {
            ToolStrip strip = toolStrip1;
            if (this.IsDisposed || this.Disposing || strip == null)
                return null;
            ToolStripItem[] Result = new ToolStripItem[strip.Items.Count];
            strip.Items.CopyTo(Result, 0);
            return Result;
        }

        #endregion
        #endregion

        #region ISerializable Members
        /// <summary>
        /// Load data from saved stream (from VWB).
        /// </summary>
        /// <param name="info">Stream from which to load</param>
        public SheetControl(SerializationInfo info, StreamingContext context)
             : this()
        {
            //
            // Example how to restore a persistent value from a stream
            //
            // m_strExampleValue = Tools.GetValue<string>(info, "ExampleValue", m_strExampleValue);
        }

        /// <summary>
        /// Save data into stream when saving VWB
        /// </summary>
        /// <param name="info">Stream in which to save</param>
        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            //
            // Example how to save a persistent value from into a stream
            //
            // info.AddValue("ExampleValue", m_strExampleValue, typeof(string));
        }
        #endregion

        private void SheetControl_Load(object sender, EventArgs e)
        {
            StatusArea.Text = "****";
            StartCmd.Enabled = false;
            StartCmd_MenuItem.Enabled = false;
            StartCmd_ToolStripItem.Enabled = false;
            PrintCmd.Enabled = false;
            PrintCmd_MenuItem.Enabled = false;
            PrintCmd_ToolStripItem.Enabled = false;
            ResultMeters.UserName = "No Values";
        }

        private void StartCmd_Click(object sender, EventArgs e)
        {
            // set all required acquisition parameters - example
            StatusArea.Text = "Setting test parameters";
            IAcquisitionSetup setup = m_MyDemoSystem.AcquisitionSetup;
            if ((setup != null) && (m_TheRecorder != null))
            {
                m_TheRecorder.Group.HighSamplingFrequency = 10000;  // sample rate 10 kHz
                setup.SweepCount = 2;
                // m_TheRecorder.Group.SweepCountEnabled = true;       // enable multiple sweep
                // m_TheRecorder.Group.SweepCount = 2;                 // 2 sweeps
                // m_TheRecorder.Group.SweepLength = 4000;             // length = 4000 samples
                // m_TheRecorder.Group.TriggerPosition = 50;           // trigger position 50%
                setup.PreTriggerInSeconds  = 2000 / m_TheRecorder.Group.HighSamplingFrequency;
                setup.PostTriggerInSeconds = 2000 / m_TheRecorder.Group.HighSamplingFrequency;
                setup.PostTriggerMode = CtrlPostTriggerMode.Immediate;
            }
            // start acquisition
            m_TheRecorder.Group.Run();
            PrintCmd.Enabled = false;
            PrintCmd_MenuItem.Enabled = false;
            PrintCmd_ToolStripItem.Enabled = false;
        }

        private void PrintCmd_Click(object sender, EventArgs e)
        {
            Reporting.Print();
        }

        private void RegisterComponents()
        {
            if (m_iProgram == null)
                return;
            if (ResultDisplay != null)
            {
                if (ResultDisplay.pDisplay != null)
                {
                    // Add the display to the component manager
                    m_iProgram.ComponentManager.Add(ResultDisplay.pDisplay);
                }
            }
        }

        private void UnRegisterComponents()
        {
            if (m_iProgram == null)
                return;
            if (ResultDisplay != null)
            {
                if (ResultDisplay.pDisplay != null)
                {
                    // Remove the display from the component manager
                    m_iProgram.ComponentManager.Remove(ResultDisplay.pDisplay);
                }
            }
        }

        #region Disposing
        /// <summary>
        /// Procedure to do your cleanup, this function is called from within the Dispose() method.
        /// </summary>
        private void SheetDisposed(object sender, EventArgs e)
        {
            if (IsDisposed)
                return;

            if (m_bDisposed)
                return;
            try
            {
                // Add your clean up code
                UnRegisterComponents();
                UnHookFromGroupAll();
                m_GroupAll = null;
                m_MyDemoSystem = null;
                m_TheRecorder = null;
            }
            catch
            {
            }
            m_bDisposed = true;
        }
        #endregion


    }
}